#!/bin/bash -
# libguestfs
# Copyright (C) 2012 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Test remote events in guestfish.

set -e

$TEST_FUNCTIONS
skip_if_skipped

eval "$(guestfish --listen)"

$VG guestfish --remote event close_event close "echo closed"

output="$($VG guestfish --remote list-events)"
if [ "$output" != '"close_event" (0): close: echo closed' ]; then
    echo "$0: list-events failed:"
    echo "$output"
    guestfish --remote exit
    exit 1
fi

# Test close event (RHBZ#802389).
output="$($VG guestfish --remote quit)"

if [ "$output" != "closed" ]; then
    echo "$0: close event failed:"
    echo "$output"
    exit 1
fi
