(* virt-builder
 * Copyright (C) 2013 Red Hat Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *)

type format =
  | Short
  | Long
  | Json

val list_formats : string list
(** The string representation of the available formats. *)

val list_format_of_string : string -> format
(** Convert from a string to the corresponding format.

    Throw [Invalid_argument] if the string does not match any
    valid format. *)

val list_entries : list_format:format -> sources:Sources.source list -> Index.index -> unit
